<?
/******************************************************************
Extension/Adaptation of the Netatmo IPS script from stuzi01
http://www.ip-symcon.de/forum/threads/21100-netatmo-wetterstation

This script creates categories for the Stations and
Dummy Modules for the base device and for each module.

11/10/2014 by Zapp for the IPS Community

The login function is based on the PHP SDK from Netatmo
see https://dev.netatmo.com/doc/sdk/php
and https://github.com/Netatmo/Netatmo-API-PHP

Requirements:
- NAApiClient.php from Netatmo github
- Adapt the parameters in the SETTINGS section below
- Copy this script into a Netatmo category and set a cyclic event
******************************************************************/
include("IPSInstaller.ips.php");
require_once("NAApiClient.php");
$config = array();

/************* SETTINGS ************************/

$config['client_id'] = 'Your_Netatmo_Client_ID'; 			// ersetzen
$config['client_secret'] = 'Your_Netatmo_Client_Secret'; 	// ersetzen

$username = "Your_Netatmo_Username_mail";  	//xx@yy ersetzen
$pwd = "Your_Netatmo_Password"; 			// ersetzen

// This array relates the Netatmo Measurements profiles to IPS Variable Types and Profiles
// "Netatmo Measurement Type" =>array("VarType"=> IPS Variable Type, "VarPosition"=> Position of the variable, "VarProfile"=> IPS Var Profile)
// See Netatmo API Documentation for Types of measurements: https://dev.netatmo.com/doc/methods/getmeasure
// See IPSInstaller.ips.php for Variablle Types
$varMeasType = array(
	"Temperature"		=>array("VarType"=>2, "VarPosition"=>0, "VarProfile"=>"~Temperature"),
	"Humidity"			=>array("VarType"=>1, "VarPosition"=>1, "VarProfile"=>"~Humidity"),
	"Co2"				=>array("VarType"=>1, "VarPosition"=>2, "VarProfile"=>"~Occurrence.CO2"),
	"Rain"				=>array("VarType"=>1, "VarPosition"=>0, "VarProfile"=>"~Rainfall"),
	"sum_rain"			=>array("VarType"=>1, "VarPosition"=>0, "VarProfile"=>"~Rainfall"),
	"Sp_Temperature"	=>array("VarType"=>2, "VarPosition"=>0, "VarProfile"=>"~Temperature"),
	"BoilerOn"			=>array("VarType"=>1, "VarPosition"=>0, "VarProfile"=>""),
	"BoilerOff"			=>array("VarType"=>1, "VarPosition"=>0, "VarProfile"=>"")
);
/*********** END OF SETTING *********************/

$client = login($config);

try
{
    $deviceList = $client->api("devicelist");
    //print_r($deviceList);
    
    // Extract data from Base station
    if(isset($deviceList["devices"][0])) // We have a base station
    {
        $NADeviceID		= $deviceList["devices"][0]["_id"];
        $NADeviceName	= $deviceList["devices"][0]["module_name"];
        $NAStationName	= $deviceList["devices"][0]["station_name"];
        $wifi       		= $deviceList["devices"][0]["wifi_status"];

		  $IPSStationCatID	= CreateCategory("$NAStationName",IPS_GetParent($_IPS['SELF']),0);
		  $IPSDeviceCatID 	= CreateInstance("$NADeviceName (Base Station)", $IPSStationCatID, "{485D0419-BE97-4548-AA9C-C083EB82E61E}", 0);
		  
		  // Update Wifi Status
		  SetValue(CreateVariable("Wifi Status", 1, $IPSDeviceCatID, 6, "", $_IPS['SELF']), $wifi);
        
        // Display dashboard_data
        if(isset($deviceList["devices"][0]["dahsboard_data"]))
            print_r($deviceList["devices"][0]["dashboard_data"]);
            
        // Retrieve current Data from Base Station
        getDeviceData($IPSDeviceCatID, $NADeviceID);
    }
    
    // Extract data for Modules
    foreach($deviceList["modules"] as $module) {
      //print_r($module);
      $NAModuleID		= $module["_id"];
      $NAModuleName 	= $module["module_name"];
      $NAModuleType 	= $module["type"];
      $battery       	= $module["battery_vp"];
      $rf       		= $module["rf_status"];

      switch ($NAModuleType) {
         case 'NAModule1': // Outdoor Module
			//echo "Type: Outdoor\r\n";
			$myType = "Outdoor";
            $type = "Temperature,Humidity";
            $order = 1;
            $battery_low = 4000;
            break;
         case 'NAModule4': // Additional Indoor Module
            //echo "Type: Additional Indoor \r\n";
            $myType = "Indoor";
            $type = "Temperature,Humidity,Co2";
            $order = 2;
            $battery_low = 4560;
            break;
         case 'NAModule3': // Rain gauge
            //echo "Type: Rain gauge \r\n";
            $myType = "Rain Gauge";
            $type = "Rain";
         	$order = 3;
         	$battery_low = 4000;
            break;
		case 'NATherm1': // Thermostat Module
			//echo "Type: Thermostat\r\n";
			$myType = "Thermostat";
			$type = "Temperature, Sp_Temperature, BoilerOn, BoilerOff";
			$order = 4;
			$battery_low = 3000;
			break;
         case 'NAPlug':	// Thermostat relay/plug
            //echo "Type: Thermostat Relay/Plug\r\n";
			$myType = "Thermostat Relay";
			$type = "";
			$order = 4;
			$battery_low = 3000; // ?
			break;
      }
      
		// Create or find sub-category for the module
		$IPSModuleCatID = CreateInstance("$NAModuleName ($myType)", $IPSStationCatID, "{485D0419-BE97-4548-AA9C-C083EB82E61E}", $order);
		// Update Battery State
		SetValue(CreateVariable("Battery", 0, $IPSModuleCatID, 5, "~Battery", $_IPS['SELF']), ($battery <= $battery_low));
		SetValue(CreateVariable("RF Status", 1, $IPSModuleCatID, 6, "", $_IPS['SELF']), $rf);
		
		// Get Measurements for the Module
		getModuleData($NADeviceID, $NAModuleID, $IPSModuleCatID, "max", $type);
    }
}
catch(NAClientException $ex)
{
    echo "An Error happened when retrieving devices and measurements: ".$ex->getMessage()."\n";
}

/********* FUNCTIONS **************
Adapted from Netatmo PHP SDK
***********************************/

function login($config) {
	global $username, $pwd;
	
   $client = new NAApiClient($config);

	$client->setVariable("username", $username);  //hier nichts ndern
	$client->setVariable("password", $pwd);       //hier nichts ndern
	try
	{
		$tokens = $client->getAccessToken();
		//echo $client;
		$refresh_token = $tokens["refresh_token"];
		$access_token = $tokens["access_token"];
	}
	catch(NAClientException $ex)
	{
		echo "An error happend while trying to retrieve your tokens : ".$ex->getMessage()."\n";
	}
	
	return $client;
}

function getDeviceData($IPSDeviceCatID, $NADeviceID) {
	global $client, $NADeviceName, $NAStationName;
	
   // Retrieve current mean day temperature,humidity
	$params = array("scale" =>"max",
	               "type"=>"Temperature,Humidity,Co2,Noise,Pressure",
	               "date_end"=>"last",
	               "device_id"=>$NADeviceID);
	$res = $client->api("getmeasure", $params);
	if(isset($res[0]) && isset($res[0]["beg_time"]))
	{
	   $time = $res[0]["beg_time"];
	   $t = $res[0]["value"][0][0];
	   $h = $res[0]["value"][0][1];
	   $c = $res[0]["value"][0][2];
	   $n = $res[0]["value"][0][3];
	   $p = $res[0]["value"][0][4];

	   
	   SetValue(CreateVariable("Temperature", 2, $IPSDeviceCatID, 0, "~Temperature",		$_IPS['SELF']), $t);

	   SetValue(CreateVariable("Humidity", 	1, $IPSDeviceCatID, 1, "~Humidity", 		$_IPS['SELF']), $h);

	   SetValue(CreateVariable("Co2", 			1, $IPSDeviceCatID, 3, "~Occurrence.CO2", $_IPS['SELF']), $c);

	   SetValue(CreateVariable("Noise", 		1, $IPSDeviceCatID, 4, "Noise", 				$_IPS['SELF']), $n);

	   SetValue(CreateVariable("Pressure", 	2, $IPSDeviceCatID, 2, "", 					$_IPS['SELF']), $p);
}
}

function getModuleData($NADeviceID, $NAModuleID, $IPSModuleCatID, $scale, $type) {
	global $client, $varMeasType;
	
	// Retrieve last measurements (put 1day for scale if 1 day average)
	$params = array("scale" =>"$scale",
	               	"type"=>$type,
	               	"date_end"=>"last",
	               	"device_id"=>$NADeviceID,
	               	"module_id"=>$NAModuleID);
	$res = $client->api("getmeasure", $params);

	if(isset($res[0]) && isset($res[0]["beg_time"]))
	{
	   $time = $res[0]["beg_time"];
	   
	   $types = explode(",", $type);
	   $i = 0;
	   foreach ($types as $thisType) {
		   $value = $res[0]["value"][0][$i];

		   //echo "Last Measurements at ".date('c', $time)." for module $NAModuleName: \r\n";
		   echo $thisType." $value -> VarType: ".$varMeasType[$thisType]['VarType']." - VarPosition: ".$varMeasType[$thisType]['VarPosition']." - VarProfile: ".$varMeasType[$thisType]['VarProfile']."\r\n";

			$varType = $varMeasType[$thisType]['VarType'];
			$varPosition = $varMeasType[$thisType]['VarPosition'];
			$varProfile = $varMeasType[$thisType]['VarProfile'];
			SetValue(CreateVariable("$thisType", $varType, $IPSModuleCatID, $varPosition, "$varProfile", $_IPS['SELF']), $value);

			$i++;
		}
}
}
?>
